"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const os = require("os");
const cli_ux_1 = require("cli-ux");
const constants_1 = require("../constants");
const Http = require("./http");
const Log = require("./log");
/**
 * Returns configuration from the credentials file.
 * @returns The contents of the credentials file if exists.
 */
function getCredentials() {
    let credentialsFileContents;
    try {
        credentialsFileContents = fs.readFileSync(`${os.homedir()}/${constants_1.STACKPATH_CREDENTIALSFILE_PATH}`, "utf8");
    }
    catch (err) {
        throw new Error("The StackPath CLI credentials file does not exist yet. Run `sp-serverless auth` to create this.");
    }
    return JSON.parse(credentialsFileContents);
}
exports.getCredentials = getCredentials;
/**
 * Saves the credentials file to disk.
 * @param credentials - The credential object that needs to be saved to disk.
 */
function saveCredentials(credentials) {
    fs.writeFileSync(`${os.homedir()}/${constants_1.STACKPATH_CREDENTIALSFILE_PATH}`, JSON.stringify(credentials));
    fs.chmodSync(`${os.homedir()}/${constants_1.STACKPATH_CREDENTIALSFILE_PATH}`, "0600");
}
exports.saveCredentials = saveCredentials;
/**
 * Checks if a credentials file exists.
 * @returns A boolean containing the value if the credentials file exists or not.
 */
function credentialsFileExists() {
    return fs.existsSync(`${os.homedir()}/${constants_1.STACKPATH_CREDENTIALSFILE_PATH}`);
}
exports.credentialsFileExists = credentialsFileExists;
/**
 * Creates the path to the credentials file if needed.
 */
function createCredentialsPathIfNeeded() {
    if (!fs.existsSync(`${os.homedir()}/${constants_1.STACKPATH_CREDENTIALS_HOME}`)) {
        Log.logVerbose(`${os.homedir()}/${constants_1.STACKPATH_CREDENTIALS_HOME} does not exist. Creating directory.`);
        fs.mkdirSync(`${os.homedir()}/${constants_1.STACKPATH_CREDENTIALS_HOME}`);
    }
}
exports.createCredentialsPathIfNeeded = createCredentialsPathIfNeeded;
/**
 * Gets an Access token from StackPath and save it to disk and returns it.
 * @returns {Promise<string>} - The access_token that was the result of the oauth2 exchange.
 */
async function getAccessToken() {
    const credentials = getCredentials();
    const response = await Http.request("POST", "/identity/v1/oauth2/token", {
        client_id: credentials.client_id,
        client_secret: credentials.client_secret,
        grant_type: "client_credentials"
    });
    const body = await response.json();
    // Exit if cannot retrieve an access token
    if (!response.ok) {
        cli_ux_1.default.log(`An error occurred trying to retrieve an access token: ${body.message}`);
        process.exit(1);
    }
    credentials.access_token = body.access_token;
    credentials.access_token_expiry =
        Math.round(new Date().getTime() / 1000) + body.expires_in;
    saveCredentials(credentials);
    return body.access_token;
}
exports.getAccessToken = getAccessToken;
/**
 * Check if current Access token is expired.
 * @returns {Promise<boolean>} If true, the token has expired.
 */
async function isAccessTokenExpired() {
    const credentials = getCredentials();
    return credentials.access_token && credentials.access_token_expiry
        ? credentials.access_token_expiry < Math.round(new Date().getTime() / 1000)
        : true;
}
exports.isAccessTokenExpired = isAccessTokenExpired;
