"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const errors_1 = require("@oclif/errors");
const cli_ux_1 = require("cli-ux");
const AuthService = require("../api/services/auth");
const Log = require("../api/services/log");
const LogLevel = Log.LogLevel;
/**
 * Class performing the auth command '$ sp-serverless auth'.
 */
class Auth extends command_1.Command {
    /**
     * Runs CLI command.
     */
    async run() {
        this.parsedFlags = this.parse(Auth).flags;
        if (!!this.parsedFlags && this.parsedFlags.verbose) {
            process.env.STACKPATH_LOG_LEVEL = LogLevel.VERBOSE || LogLevel.INFO;
        }
        if (!this.parsedFlags.force) {
            Log.logVerbose(`Checking if credentials file exists`);
        }
        if (AuthService.credentialsFileExists() && !this.parsedFlags.force) {
            const overwriteFile = await cli_ux_1.default.prompt("It seems a credentials file already exists. Continuing will overwrite its values. Continue? y/n");
            if (overwriteFile !== "y") {
                throw new errors_1.CLIError("Aborting...");
            }
            Log.logVerbose(`Credentials file will be overwritten. Continuing...`);
        }
        const clientID = this.parsedFlags.client_id || (await cli_ux_1.default.prompt("StackPath Client ID"));
        const clientSecret = this.parsedFlags.client_secret ||
            (await cli_ux_1.default.prompt("StackPath Client Secret", {
                type: "hide"
            }));
        AuthService.createCredentialsPathIfNeeded();
        Log.logVerbose(`Saving credentials to disk.`);
        AuthService.saveCredentials({
            client_id: clientID,
            client_secret: clientSecret
        });
        cli_ux_1.default.action.start("Getting Access token...");
        await AuthService.getAccessToken();
        cli_ux_1.default.action.stop("Received Access token.");
        this.log("Your credentials have been configured.");
    }
}
Auth.description = "Configures StackPath's authentication details.";
Auth.examples = [`$ sp-serverless auth`];
Auth.flags = {
    client_id: command_1.flags.string({
        char: "c",
        description: "StackPath Client ID used to authenticate with"
    }),
    client_secret: command_1.flags.string({
        char: "s",
        description: "StackPath Client Secret used to authenticate with"
    }),
    force: command_1.flags.boolean({
        char: "f",
        description: "Set this to always overwrite current credential file, defaults to false.",
        default: false
    }),
    help: command_1.flags.help({ char: "h" }),
    verbose: command_1.flags.boolean({
        char: "v",
        description: "Turns on verbose logging. Defaults to false",
        default: false
    })
};
Auth.args = [];
exports.default = Auth;
